#! /usr/bin/perl

#############################################################################
# slashpluck.pl
#
# $Id: slashpluck.pl,v 1.7 2002/04/05 17:04:38 jday Exp $
# 
# Copyright (C) 2002 by Jason Day <jasonday@worldnet.att.net>
#
# Fetches headlines and stories from Slashdot and formats them in a friendly way
# for Plucker (http://www.plkr.org/).  The headlines and articles are stored as
# html files in a directory accessible by plucker.
#
# The timestamp of the headlines file is checked first, so that fetching occurs
# no more than once every 30 minutes (this was a guideline specified in the
# slashdot FAQ, though the most recent FAQ doesn't seem to have it).
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#############################################################################

use strict;
use vars qw($pluckerdir $directory $web_directory $threshold $headlinesfile
            $DEBUG);
use Socket;

# The plucker home directory.  Uncomment one and customize as needed.
# This should be fine for UNIX
$pluckerdir = "$ENV{HOME}/.plucker";
# This should work for windows
#$pluckerdir = "c:/Program Files/Plucker/Default.DB";

# Article threshold.  Only articles with this score or higher will be
# retrieved.  I suggest using 3 or higher, unless you have lots of room
# on your PDA.
$threshold = 3;

#$DEBUG++;

#############################################################################
# End of configuration section
#############################################################################

# The directory where the html files are stored.
$directory = "$pluckerdir/slashpluck";

# The same directory as it appears to the browser.
$web_directory = "plucker:/slashpluck";

# Filename for the headlines.  There's not really any reason to change this.
$headlinesfile = "$directory/headlines.html";

#############################################################################

# Make sure the plucker directory exists
die "Plucker directory $pluckerdir does not exist. Check your configuration.\n"
    unless (-d $pluckerdir);

# Create the slashpluck directory if necessary
unless (-d $directory) {
    print "Creating directory $directory\n";
    mkdir $directory or die "Error creating directory $directory\n";
}

# Make sure we can write to the directory
die "Cannot write to directory $directory. Check your configuration.\n"
    unless (-w $directory);

# Check the modification time of the headlines file.  If it's more than
# 30 minutes then refetch the headlines and articles from slashdot.
if (-e $headlinesfile) {
    my $mtime = (stat ($headlinesfile))[9];
    my $diff = time - $mtime;
    if ($diff >= 1800) {
        &fetch();
    }
    else {
        print "$headlinesfile was last updated $diff seconds ago, fetch skipped.\n";
    }
}
else {
    # First-time run, so go fetch
    &fetch();
}


#############################################################################

sub fetch() {
    my $update_time = localtime(time);
    open (OUTFILE, ">$headlinesfile");
    print OUTFILE <<EOF;
<html>
<head>
<title>Slashdot Headlines</title>
</head>
<body bgcolor=white>
<center>
<img src="$web_directory/topicslashdot.gif" width=100 height=34 border=0 alt="Slashdot.org">
<br>
<b>News for Nerds: Stuff that Matters.</b>
<p>
Last updated: $update_time
</center>
EOF

    print "Fetching slashdot.xml\n";
    openURL ('XMLFILE', "http://slashdot.org/slashdot.xml")
        or die "Error fetching slashdot.xml";
    my $title = 0;
    my $url = 0;
    my $i = 0;
    my $num = 0;
    my ($line, $buffer);
    while (<XMLFILE>) {
        if (/.*(\<story\>).*/) {
            $title = 1;     
            next;
        }
        if ($title == 1) {
            if (/.*\<title\>(.+)\<\/title\>.*/) {
                $buffer = $1;
                $title = 0;
                $url = 1;
                next;
            }
        }
        if ($url == 1) {
            if (/.*\<url\>(.+)\<\/url\>.*/) {
                $line = $1;
                $url = 0;
                $i = 1;
            }
        }
        if ($i == 1) {
            $num++;
            print "Fetching article: $line\n";
            openURL ('ARTFILE', "$line&light=1&noboxes=1&threshold=$threshold")
                or die "Error fetching article";
            open (FILE, ">$directory/$num.html")
                or die "Error opening article file";
            print FILE getHeader ('ARTFILE');
            print FILE getStory ('ARTFILE');
            print FILE getComments ('ARTFILE');
            close FILE;
            close ARTFILE;
            print OUTFILE "<p><a href=\"$web_directory/$num.html\">$buffer</a>\n";
            $i = 0;
        }
    }
    close (XMLFILE);
    
    print OUTFILE <<EOF;
</body>
</html>
EOF
    
    close (OUTFILE);
}

sub getHeader() {
    no strict "refs";

    my ($handle) = @_;
    my $done = 0;
    my $buffer = "";
    
    print "getHeader() - $handle\n" if $DEBUG;
    while (<$handle>) {
        if ($done) {
            last;
        }

        if (/<body>/i) {
            $done = 1;
        }

        $buffer .= $_;
    }
    
    print "getHeader() - returning $buffer\n" if $DEBUG;
    return $buffer;
}

sub getStory() {
    no strict "refs";

    my ($handle) = @_;
    my $skip = 1;
    my $buffer = "";

    print "getStory() - $handle\n" if $DEBUG;
    while (<$handle>) {
        if ($skip) {
            if (/search\.pl\?topic/) {
	        $skip = 0;
	    }

	    next;
	}
	#elsif (/^[ 	]*\&lt;.*slashdot.org/i) {
	elsif (/^[ 	]*<p>\&nbsp;<\/td><td>\&nbsp;/i) {
	    last;
	}

        s/<a href=[^>]*slashdot.org[^>]*>([^<]*)<\/a>/$1/i;
        $buffer .= $_;
    }
    
    print "getStory() - returning $buffer\n" if $DEBUG;
    return $buffer;
}

sub getComments() {
    no strict "refs";

    my ($handle) = @_;
    my $skip = 1;
    my $buffer = "<hr>\n";

    while (<$handle>) {
        if (m{^ *</font></td></tr><tr>}i) {
            last;
        }
    }

    while (<$handle>) {
        if ($skip) {
            if (/<a name=[^>]*><h4>/i) {
	        $skip = 0;
	    }
            elsif (/<\/form>/i) {
                $skip = 0;
                $buffer .= "<p>\n";
                next;
            }
            else {
                next;
            }
        }
        elsif (m{^[ 	]*</td>[ 	]*$}i) {
            $skip = 1;
            $buffer .= "<hr>";
            next;
        }

        s/<a href=[^>]*slashdot.org[^>]*>([^<]*)<\/a>/$1/i;
        $buffer .= $_;
    }
    
    return $buffer;
}

sub openURL {
    no strict "refs";

    my ($handle, $url) = @_;
    print "openURL() - URL: $url\n" if $DEBUG;

    # attempt to parse the URL
    $url =~ m|(\w+)://([^/:]+)(:\d*)?([^#]*)| || return 0;

    my $protocol = $1;
    my $server = $2;
    my $port = $3;
    my $file = $4;

    # only handle HTTP for now
    return 0 unless ($protocol =~ /http/i);

    # set port to 80 if it wasn't given
    $port = 80 unless (defined $port);

    # strip the : from the port
    $port =~ s/^://;

    # set file to / if not given
    $file = "/" if (length $file == 0);

    print "openURL() - server: $server\n" if $DEBUG;
    print "openURL() - port: $port\n" if $DEBUG;
    print "openURL() - file: $file\n" if $DEBUG;

    # setup the socket
    my $proto = getprotobyname ("tcp");
    socket ($handle, PF_INET, SOCK_STREAM, $proto);
    my $sin = sockaddr_in ($port, inet_aton ($server));
    connect ($handle, $sin) || return 0;

    my $old_h = select ($handle);
    $| = 1;
    select ($old_h);

    print $handle "GET $file HTTP/1.0\r\n";
    #print $handle "Accept: */*\n\n";
    print $handle "\r\n";

    # get the HTTP response
    my $response = <$handle>;
    print "openURL() - Response: $response\n" if $DEBUG;

    # get the headers
    while (<$handle> =~ /(\S+):\s+(.+)/) {
        print "openURL() - Header: $1: $2\n" if $DEBUG;
    }

    return 1;
}
